function [thetaopt,alpha,sigma2,R1,ResR2I,R2I]=FunctionalGPFit0(Din1,Din2,Yin,Noin,start)
%checked#2
%Inputs: Din1=the set of points M at which the functional response is observed, Din2=the experiment design, Yin=data in the format of a matrix as in Y_* in Equation (11), Noin=choice of correlation function (1 for Matrn correlation function in Equation (9)), start=starting point for optimizing likelihood function (leave empty to use default starting points).
%Outputs: [thetaopt,alpha,sigma2,R1,ResR2I,R2I]=Several vectors and matrices to be used by FunctionalGPPredict0 to compute point predictions and prediction intervals, thetaopt=maximum likelihood estimate of correlation parameters, alpha=maximum likelihood estimate of mean, sigma2= maximum likelihood estimate of variance. 

global D1 D2 Y n No n1 n2 d1 d2 scaling C1 C2

No=Noin;
D1=Din1;
D2=Din2;
Y=Yin;
[n1, d1]=size(D1);
[n2, d2]=size(D2);
n=n1*n2;
d=d1+d2;
if(sum(abs(size(Y)-[n1 n2]))>10^-6)
    display('error')
    return    
end
scaling=[range(D1,1) range(D2,1)];

C1=cell(1,d1);
for i=1:d1
    C1{i}=abs(repmat(D1(:,i),1,n1)-repmat(D1(:,i)',n1,1));
end
C2=cell(1,d2);
for i=1:d2
    C2{i}=abs(repmat(D2(:,i),1,n2)-repmat(D2(:,i)',n2,1));
end

if(nargin==5)
    if(No==0)
        [paropt, fval, exitflag]=patternsearch(@Obj,start,[],[],[],[],-12*ones(1,d),12*ones(1,d));         
    else
        [paropt, fval, exitflag]=patternsearch(@Obj,start,[],[],[],[],-75*ones(1,d),30*ones(1,d)); 
    end
else
if(No==0)
    [paropt, fval, exitflag]=patternsearch(@Obj,[norminv((0.75.^(1./scaling.^2)).*ones(1,d))],[],[],[],[],-12*ones(1,d),12*ones(1,d)); 
elseif(No==1)
    [paropt, fval, exitflag]=patternsearch(@Obj,[log(1.04028096570006*ones(1,d).*scaling)],[],[],[],[],-75*ones(1,d),30*ones(1,d));
elseif(No==2)
    [paropt, fval, exitflag]=patternsearch(@Obj,log(0.70889432235799*ones(1,d).*scaling),[],[],[],[],-75*ones(1,d),30*ones(1,d)); 
end
end

if((exitflag<=0))
    display('Likelihood optimization failure: algorithm did not converge.')   
end
if(No==0)
    thetaopt=normcdf(paropt);
elseif(No==1)
    thetaopt=exp(paropt);        
elseif(No==2)
    thetaopt=exp(paropt);        
end

display('Optimal values of GP parameters')
disp(thetaopt)
R1=CompCorr1(thetaopt(1:d1));
R2=CompCorr2(thetaopt((d1+1):(d1+d2)));

R1I=invandlogdet(R1);
R2I=invandlogdet(R2);
alpha=ones(1,n1)*R1I*Y*R2I*ones(n2,1)/(sum(sum(R1I))*sum(sum(R2I)));
Res=Y-ones(n1,n2)*alpha;
ResR2I=Res*R2I;
RIRes=R1I*ResR2I;
RIRes=RIRes(:);
sigma2=(Res(:)')*RIRes/n;

function Objective=Obj(par)
global D1 D2 Y n No n1 n2 d1 d2 scaling

if(sum(isnan(par))>0)
    Objective=Inf;
    return
end
if(No==0)
    r=normcdf(par);
    if((sum(r<(0.0001.^(1./scaling.^2)))>0)||(sum(r>(0.9999.^(1./scaling.^2)))>0))
        Objective=Inf;
        return
    end
elseif(No==1)
    r=exp(par);    
    if((sum(r<(0.0850602606088632.*scaling))>0)||(sum(r>(70.3767525477388.*scaling))>0))
        Objective=Inf;
        return        
    end      
elseif(No==2)
    r=exp(par);
    if((sum(r<(0.0738774798349666.*scaling))>0)||(sum(r>(40.8218063231753.*scaling))>0))
        Objective=Inf;
        return        
    end
end

R1=CompCorr1(r(1:d1));
R2=CompCorr2(r((d1+1):(d1+d2)));

[R1I,LD1]=invandlogdet(R1);
[R2I,LD2]=invandlogdet(R2);
LDR=(n2)*LD1+(n1)*LD2;
alpha=ones(1,n1)*R1I*Y*R2I*ones(n2,1)/(sum(sum(R1I))*sum(sum(R2I)));

Res=Y-ones(n1,n2)*alpha;
RIRes=R1I*Res*R2I;
RIRes=RIRes(:);
sigma2=(Res(:)')*RIRes/n;
Objective=n*log(max(sigma2,0))+LDR;
if(isnan(Objective))
    Objective=Inf;
end

function Corr=CompCorr1(r)
global No C1 d1
if(No==0)
    Corr=1;
    for i=1:d1
        Corr=Corr.*(r(i).^(C1{i}.^2));
    end
elseif(No==1)
    Corr=1;
    for i=1:d1
        rho1=C1{i}./r(i);
        Corr=Corr.*((exp(-rho1)).*(rho1+1));
    end
elseif(No==2)
    Corr=1;
    for i=1:d1
        rho1=C1{i}./r(i);
        Corr=Corr.*((exp(-rho1)/3).*(rho1.^2+3*rho1+3)); 
    end
end

function Corr=CompCorr2(r)
global No C2 d2
if(No==0)
    Corr=1;
    for i=1:d2
        Corr=Corr.*(r(i).^(C2{i}.^2));
    end
elseif(No==1)
    Corr=1;
    for i=1:d2
        rho1=C2{i}./r(i);
        Corr=Corr.*((exp(-rho1)).*(rho1+1));
    end
elseif(No==2)
    Corr=1;
    for i=1:d2
        rho1=C2{i}./r(i);
        Corr=Corr.*((exp(-rho1)/3).*(rho1.^2+3*rho1+3)); 
    end
end