function [thetaopt,sigma2,ResR2I,R2I,R1]=FunctionalGPFitWave(Din1,Din2,Yin,Uin)
%checked#1
%Inputs: Din1=the set of points M at which the functional response is observed, Din2=the experiment design, Yin=data in the format of a matrix as in Y_* in Equation (11), Uin=regression matrix computed with design D_1 such that U(1,z)^T is the mean of the functional response, as given by (21).
%Outputs: [thetaopt,sigma2,ResR2I,R2I,R1]=Several vectors and matrices to be used by FunctionalGPPredictWave to compute point predictions and prediction intervals, thetaopt=maximum likelihood estimate of parameters in the covariance function (23), except \sigma_0^2. sigma2=maximum likelihood estimate of \sigma_0^2.

global D1 D2 Y U n n1 n2 d1 d2 scaling C1 C2 nugget Res

D1=Din1; D2=Din2;
Y=Yin; U=Uin; 
[n1, d1]=size(D1);
[n2, d2]=size(D2);
n=n1*n2;
if(sum(abs(size(Y)-[n1 n2]))>10^-6)
    display('error')
    return    
end
Maxx=max(D2,[],1);
D2=D2./repmat(Maxx,n2,1);
U=U.*repmat([1 Maxx(3:end)],n1,1);
miu=U*[ones(n2,1) D2(:,3:end)]';
Res=Y-miu;

C1=cell(1,d1);
for i=1:d1
    C1{i}=abs(repmat(D1(:,i),1,n1)-repmat(D1(:,i)',n1,1));
end
C2=cell(1,d2);
for i=1:d2
    C2{i}=abs(repmat(D2(:,i),1,n2)-repmat(D2(:,i)',n2,1));
end

scaling=[range(D1,1) range(D2(:,1:2),1)];
nugget=0; 

[paropt, fval, exitflag]=patternsearch(@Obj,[log(1.04028096570006*ones(1,d1).*scaling(1:d1)) log([1.04028096570006*ones(1,2).*scaling((d1+1):(d1+2)),10]) 10*[1 1 1 1]],[],[],[],[],-75*ones(1,8),30*ones(1,8));
if((exitflag<=0))
    display('Likelihood optimization failure: algorithm did not converge.')   
end
thetaopt=exp(paropt); 

display('Optimal values of GP parameters')
disp(thetaopt)
R1=CompCorr1(thetaopt(1:d1));
R1=R1+nugget*speye(n1);
R2=CompCorr2(thetaopt((d1+1):(end)));
R1I=invandlogdet(R1); R2I=invandlogdet(R2);
A=Res'*R1I;
ResR2I=Res*R2I;
sigma2=sum(sum(A'.*ResR2I))/n

function Objective=Obj(par)
global D2 Y U n n1 n2 d1 scaling nugget Res

r=exp(par);
if((sum(isnan(par))>0)||(sum(isinf(r))>0))
    Objective=Inf;
    return
end

if((sum(r(1:(d1+2))<(0.0850602606088632.*scaling(1:(d1+2))))>0)||(sum(r(1:(d1+2))>(70.3767525477642.*scaling(1:(d1+2))))>0))
   Objective=Inf;
   return        
end

R1=CompCorr1(r(1:d1));
R1=R1+nugget*speye(n1);
R2=CompCorr2(r((d1+1):(end)));
[R1I,LD1]=invandlogdet(R1);
[R2I,LD2]=invandlogdet(R2);
LDR=(n2)*LD1+(n1)*LD2;
A=Res'*R1I; 
B=Res*R2I;
sigma2=sum(sum(A'.*B))/n;
Objective=n*log(max(sigma2,0))+LDR;

function Corr=CompCorr1(r)
global C1 d1
Corr=1;
for i=1:d1
    rho1=abs(C1{i})./r(i);
    Corr=Corr.*(exp(-rho1).*(rho1+1));
end

function Corr=CompCorr2(r)
global C2 D2 d2
Corr=1;
for i=1:2
    rho1=abs(C2{i})./r(i);
    Corr=Corr.*(exp(-rho1).*(rho1+1));
end

rho2=sqrt(C2{3}.^2+C2{4}.^2+C2{5}.^2+C2{6}.^2)./r(3);
Corr=Corr.*(r(4)*D2(:,3)*D2(:,3)'+r(5)*D2(:,4)*D2(:,4)'+r(6)*D2(:,5)*D2(:,5)'+r(7)*D2(:,6)*D2(:,6)'+exp(-rho2).*(rho2+1));