function [thetaopt,alpha,Phi,ResR2I,R2I]=MultiResponseGPFit(Din2,Zin,Noin)
%checked#1
%Inputs: Din2=the experiment design, Zin=data in the format of a matrix as in Y_* in Equation (11), Noin=choice of correlation function (1 for Matrn correlation function in Equation (9)).
%Outputs: [thetaopt,alpha,Phi,ResR2I,R2I]=Several vectors and matrices to be used by MultiResponseGPPredict to compute point predictions and prediction intervals, thetaopt=maximum likelihood estimate of correlation parameters, alpha=maximum likelihood estimate of the vector of prior means, Phi= maximum likelihood estimate of prior covariance matrix for multivariate response at fixed design point. 
global D2 Z No n1 n2 d2 scaling C2

No=Noin;
D2=Din2;
Z=Zin;
n1=size(Z,1);
[n2, d2]=size(D2);
scaling=range(D2,1);

C2=cell(1,d2);
for i=1:d2
    C2{i}=abs(repmat(D2(:,i),1,n2)-repmat(D2(:,i)',n2,1));
end

if(No==0)
    [paropt, fval, exitflag]=patternsearch(@Obj,[norminv((0.75.^(1./scaling.^2)).*ones(1,d2))],[],[],[],[],-12*ones(1,d2),12*ones(1,d2));
elseif(No==1)
    [paropt, fval, exitflag]=patternsearch(@Obj,[log(1.04028096570006*ones(1,d2).*scaling)],[],[],[],[],-75*ones(1,d2),30*ones(1,d2));      
%    [paropt, fval, exitflag]=ga(@Obj,d2,[],[],[],[],-75*ones(1,d2),30*ones(1,d2));      
elseif(No==2)
    [paropt, fval, exitflag]=patternsearch(@Obj,log(0.70889432235799*ones(1,d2).*scaling),[],[],[],[],-75*ones(1,d2),30*ones(1,d2));    
end

if((exitflag<=0))
    display('Likelihood optimization failure: algorithm did not converge.')   
end
if(No==0)
    thetaopt=normcdf(paropt);
elseif(No==1)
    thetaopt=exp(paropt);  
elseif(No==2)
    thetaopt=exp(paropt);          
end
display('Optimal values of GP parameters')
disp(thetaopt)

R2=CompCorr(thetaopt);
[R2I]=invandlogdet(R2);
alpha=Z*R2I*ones(n2,1)/(sum(sum(R2I)));
Res=(Z-alpha*ones(1,n2));
ResR2I=Res*R2I;
Phi=ResR2I*Res'/(n2);

function Objective=Obj(par)
global Z No n1 n2 scaling

if(sum(isnan(par))>0)
    Objective=Inf;
    return
end
if(No==0)
    r=normcdf(par);
    if((sum(r<(0.0001.^(1./scaling.^2)))>0)||(sum(r>(0.9999.^(1./scaling.^2)))>0))
        Objective=Inf;
        return
    end
elseif(No==1)
    r=exp(par);    
    if((sum(r<(0.0850602606088632.*scaling))>0)||(sum(r>(70.3767525477412*10^12.*scaling))>0))
        Objective=Inf;
        return        
    end      
elseif(No==2)
    r=exp(par);
    if((sum(r<(0.0738774798349666.*scaling))>0)||(sum(r>(40.8218063231678.*scaling))>0))
        Objective=Inf;
        return        
    end
end

R2=CompCorr(r);
[R2I,LD2]=invandlogdet(R2);
alpha=Z*R2I*ones(n2,1)/(sum(sum(R2I)));
Res=(Z-alpha*ones(1,n2));
Phi=Res*R2I*Res'/(n2);
Objective=(n2)*log(max(prod(svd(Phi)),0))+n1*LD2;

function Corr=CompCorr(r)
global No d2 C2
if(No==0)
    Corr=1;
    for i=1:d2
        Corr=Corr.*(r(i).^(C2{i}.^2));
    end
elseif(No==1)
    Corr=1;
    for i=1:d2
        rho1=C2{i}./r(i);
        Corr=Corr.*((exp(-rho1)).*(rho1+1));
    end
elseif(No==2)
    Corr=1;
    for i=1:d2
        rho1=C2{i}./r(i);
        Corr=Corr.*((exp(-rho1)/3).*(rho1.^2+3*rho1+3)); 
    end
end