function [thetaopt,miu,Phi,ResR2I,R2I]=MultiResponseGPFitElasticity2(Din2,Zin,Uin)
%checked#1
%Inputs: Din2=the experiment design, Zin=data in the format of a matrix as in Y_* in Equation (11), Uin=regression matrix computed with design D_1 such that U(1,z)^T is the mean of the functional response, as given by (21).
%Outputs: [thetaopt,miu,Phi,ResR2I,R2I]=Several vectors and matrices to be used by MultiResponseGPPredictElasticity2 to compute point predictions and prediction intervals, thetaopt=maximum likelihood estimate of parameters in the covariance function (23), except the prior covariance matrix for multivariate response \Phi, miu=matrix of computed prior means at design points, where each column gives the prior mean of the multivariate response at a design point, \Phi=maximum likelihood estimate of \Phi.
global D2 Z U n1 n2 d2 scaling C2 Res nugget

D2=Din2; Z=Zin; U=Uin;
n1=size(Z,1);
[n2, d2]=size(D2);
if(sum(abs(size(Z)-[n1 n2]))>10^-6)
    display('error')
    return    
end
Maxx=max(D2,[],1);
D2=D2./repmat(Maxx,n2,1);
U=U.*repmat([1 Maxx(3:end)],n1,1);
miu=U*[ones(n2,1) D2(:,3:end)]';
Res=(Z-miu);

C2=cell(1,d2);
for i=1:d2
    C2{i}=abs(repmat(D2(:,i),1,n2)-repmat(D2(:,i)',n2,1));
end
scaling=[range(D2(:,1:2))];
nugget=0; 

[paropt, fval, exitflag]=patternsearch(@Obj,[log(1.04028096570006*ones(1,2).*scaling) 1*ones(1,4) log(10)],[],[],[],[],-75*ones(1,7),30*ones(1,7)); 
%[paropt, fval, exitflag]=ga(@Obj,7,[],[],[],[],-75*ones(1,7),30*ones(1,7));

if((exitflag<=0))
    display('Likelihood optimization failure: algorithm did not converge.')   
end
thetaopt=exp(paropt);

display('Optimal values of GP parameters')
disp(thetaopt)

R2=CompCorr(thetaopt);
R2=R2+nugget*mean(diag(R2))*speye(n2);
[R2I]=invandlogdet(R2);
ResR2I=Res*R2I;
Phi=ResR2I*Res'/(n2);

function Objective=Obj(par)
global n1 n2 scaling Res nugget

if(sum(isnan(par))>0)
    Objective=Inf;
    return
end
r=exp(par);    
if((sum(r(1:2)<(0.0850602606088632.*scaling))>0)||(sum(r(1:2)>(70.3767525477512.*scaling))>0))
    Objective=Inf;
    return        
end  

R2=CompCorr(r);
R2=R2+nugget*mean(diag(R2))*speye(n2);
[R2I,LD2]=invandlogdet(R2);
Phi=Res*R2I*Res'/(n2);
Objective=(n2)*log(max(prod(svd(Phi)),0))+n1*LD2;

function Corr=CompCorr(r)
global D2 C2

Corr=1;
for i=1:2
    rho1=C2{i}./r(i);
    Corr=Corr.*(exp(-rho1).*(rho1+1));
end
rho2=sqrt(C2{3}.^2+C2{4}.^2+C2{5}.^2+C2{6}.^2)./r(7);
Corr=Corr.*(r(3)*D2(:,3)*D2(:,3)'+r(4)*D2(:,4)*D2(:,4)'+r(5)*D2(:,5)*D2(:,5)'+r(6)*D2(:,6)*D2(:,6)'+exp(-rho2).*(rho2+1));
