function [Zpred, UCL, LCL, PosCov]=MultiResponseGPPredict(Din2,Noin,x,thetaopt,alpha,Phi,ResR2I,R2I)
%checked#1
%Inputs: [Din2, Noin]=same as MultiResponseGPFit, x=input setting at which prediction of multivariate response is desired, [thetaopt,alpha,Phi,ResR2I,R2I]=the outputs of MultiResponseGPFit.
%Outputs: Zpred=vector of point predictions, UCL=vector of upper prediction/credible interval limits (0.99 quantile), LCL=vector of lower prediction/credible interval limits (0.01 quantile), PosCov=posterior covariance matrix of the multivariate response at x.
global No

No=Noin;
D2=Din2;
[n2]=size(D2,1);
n1=size(alpha,1);
m=size(x,1);
rx=zeros(n2,m);
for i=1:n2
    for j=1:m
        rx(i,j)=CompCorr(D2(i,:),x(j,:),thetaopt);
    end
end
dev=ResR2I*rx;
Zpred=repmat(alpha,1,m)+dev;

R4=zeros(m,m);
for i=1:m
    for j=i:m
        R4(i,j)=CompCorr(x(i,:),x(j,:),thetaopt);
    end
end
for i=2:m
    for j=1:(i-1)    
        R4(i,j)=R4(j,i); 
    end
end

PosCov=kron(R4-(rx')*R2I*rx,Phi);

RMSE=sqrt(max(diag(PosCov),0));
LCL=Zpred-norminv(0.99)*reshape(RMSE,n1,m);
UCL=Zpred+norminv(0.99)*reshape(RMSE,n1,m);

function Corr=CompCorr(x1,x2,r)
global No
if(No==0)
    Corr=prod(r.^(abs(x1-x2).^2));
elseif(No==1)
    rho=x1-x2;    
    rho1=abs(rho)./r;
    Corr=prod((exp(-rho1)).*(rho1+1));
elseif(No==2)
    rho=x1-x2;
    rho1=abs(rho)./r;
    Corr=prod((exp(-rho1)/3).*(rho1.^2+3*rho1+3)); 
end