function [Zpred, UCL, LCL, PosCov]=MultiResponseGPPredictElasticity2(Din2,x,thetaopt,U,Phi,ResR2I,R2I)
%checked#2
%Inputs: Din2=same as MultiResponseGPFitElasticity2, x= input setting at which prediction of multivariate response is desired, U=same as Uin in MultiResponseGPFitElasticity2, [thetaopt,Phi,ResR2I,R2I]=the outputs of MultiResponseGPFitElasticity2.
%Outputs: Zpred=vector of point predictions, UCL=vector of upper prediction/credible interval limits (0.99 quantile), LCL=vector of lower prediction/credible interval limits (0.01 quantile), PosCov=posterior covariance matrix of the multivariate response at x.

D2=Din2;
[n2]=size(D2,1);
n1=size(Phi,1);
m=size(x,1);
Maxx=max(D2,[],1);
D2=D2./repmat(Maxx,n2,1);
x=x./Maxx;
U=U.*repmat([1 Maxx(3:end)],n1,1);
rx=zeros(n2,m);
for i=1:n2
    for j=1:m
        rx(i,j)=CompCorr(D2(i,:),x(j,:),thetaopt);
    end
end
dev=ResR2I*rx;
miu=U*([ones(m,1) x(:,3:end)]');
Zpred=miu+dev;

R4=zeros(m,m);
for i=1:m
    for j=i:m
        R4(i,j)=CompCorr(x(i,:),x(j,:),thetaopt);
    end
end
for i=2:m
    for j=1:(i-1)    
        R4(i,j)=R4(j,i); 
    end
end

PosCov=kron(R4-(rx')*R2I*rx,Phi);

RMSE=sqrt(max(diag(PosCov),0));
LCL=Zpred-norminv(0.99)*reshape(RMSE,n1,m);
UCL=Zpred+norminv(0.99)*reshape(RMSE,n1,m);

function Corr=CompCorr(x1,x2,r)

Corr=1;
rho1=abs(x1(1:2)-x2(1:2))./r(1:2);
Corr=Corr*prod(exp(-rho1).*(rho1+1));
rho2=sqrt(sum(x1(3:end)-x2(3:end)).^2)/r(7);
Corr=Corr*(r(3)*x1(3)*x2(3)+r(4)*x1(4)*x2(4)+r(5)*x1(5)*x2(5)+r(6)*x1(6)*x2(6)+exp(-rho2).*(rho2+1));
