function [y,points,model]=NonlinearHeat(par)
global parbeta parh parTInf park paru
%checked#1
%Input: par=input settings (x,z) for simulator.
%Output: y=response defined in Section 5.2, i.e., temperature at 101 equally spaced points along a vertical line on the inner cylindrical surface of the gripper pivot, points= spatial coordinates at which functional response is observed.

%thermal conductivity
%figure(21),plot([-125 -100 -80 -30 25 100 200 300 400]+273.15,[45.3 48.7 51.2 57.3 63.2 70 76.9 81.4 83.5],'o')
%hold on,plot([-125 -100 -80 -30 25 100 200 300 400]+273.15,[67.9 71.2 73.7 79.3 84.7 90.5 95.8 98.4 98.6],'o')
%thermal conductivity range = [0 20.29]
park=par(1); parbeta=5.6704*10^-8; parh=25;
g1=(100+50*par(2)); g2=(100+50*par(3)); paru=15+15*par(4); parTInf=288.15;

model = createpde(1);
importGeometry(model,'Gripper Pivot.stl');

%figure(1),
%h = pdegplot(model,'FaceLabels','on');
%view(30,30);
%title('Bracket with Face Labels')
%h(1).FaceAlpha = 0.5;

applyBoundaryCondition(model,'dirichlet','Face',43,'u',@ufunc);
applyBoundaryCondition(model,'neumann','Face',38,'g',g1,'q',0);
applyBoundaryCondition(model,'neumann','Face',44,'g',g2,'q',0);
others=setdiff(1:model.Geometry.NumFaces,[38,43,44]);
applyBoundaryCondition(model,'neumann','Face',others,'g',@gfunc,'q',@qfunc);

a = 0; f = 0;
generateMesh(model,'Hmax',0.09);
specifyCoefficients(model,'m',0,'d',0,'c',@cfunc,'a',a,'f',f);
u = solvepde(model);

indices=find(abs(model.Mesh.Nodes(3,:)-max(model.Mesh.Nodes(3,:)))<10^-6);
distances=pdist2(model.Mesh.Nodes(1:2,indices)',[0.0272666222883294 1.52453907369421]);
mindist=min(distances);
indices2=find(abs(distances-mindist)<0.01);
circlepoints=model.Mesh.Nodes(1:3,indices(indices2))';
indices3=find(pdist2(model.Mesh.Nodes(1:2,:)',circlepoints(1,1:2))<0.1);
zl=min(model.Mesh.Nodes(3,indices3)); zu=circlepoints(1,3);
z=zl:((zu-zl)/100):zu;
points=[repmat(circlepoints(1,1:2),101,1) z'];
y = interpolateSolution(u,points(:,1),points(:,2),points(:,3));

function c = cfunc(region,state)
global park

if(sum(sum(isnan(state.u)))>0)
    c=NaN*ones(1,length(region.x));
    return
end
c=park+pchip([-125 -100 -80 -30 25 100 200 300 400]+273.15,[45.3 48.7 51.2 57.3 63.2 70 76.9 81.4 83.5],state.u);
c=c*39.3701;

function u=ufunc(region,state)
global paru
%sensorpoints=[-1 1; -1 1.5; -1 2; 1 1; 1 1.5; 1 2];
%vq3 = griddata(x,y,v,0.5,0.5,'v4')
u=288.15+paru*ones(1,length(region.x));

function g = gfunc(region,state)
global parbeta parh parTInf

if(sum(sum(isnan(state.u)))>0)
    g=NaN*ones(1,length(region.x));
    return
end
g=(parbeta*(parTInf^4-state.u.^4)+parh*parTInf)*ones(1,length(region.x));

function q = qfunc(region,state)
global parh

q=parh*ones(1,length(region.x));