function [upoint,point,u,mesh]=ThreeDimElasticity(par)
%checked#1
%Input: par=input settings (x,z) for simulator.
%Output: upoint=vector of displacements in each coordinate direction at the point (s_1,s_2,s_3)=(0.215,0.1,0.1055). 
%Note that the displacement in s_2 direction (second component) is ignored in our modeling exercise.

global parg1 parg2
rho=7850;
g=9.8;
parE=par(1); parnu=par(2); parg1=par(3); parg2=par(4:end);
model = createpde(3);
importGeometry(model,'BracketWithHole.stl');

%figure(1),
%h = pdegplot(model,'FaceLabels','on');
%view(30,30);
%title('Bracket with Face Labels')
%h(1).FaceAlpha = 0.5;

applyBoundaryCondition(model,'Face',4,'u',[0,0,0]);
applyBoundaryCondition(model,'Face',7,'g',@g1fun);
applyBoundaryCondition(model,'Face',11,'g',@g2fun);
E = parE*10^9;
%E = 200e9;
nu = parnu;
%nu = 0.3;
c = elasticityC3D(E,nu);
a = 0;
f = [0;0;-rho*g];
generateMesh(model,'Hmax',0.0016,'GeometricOrder','linear');
%generateMesh(model,'Hmax',0.008);
u = assempde(model,c,a,f);
%result = createPDEResults(model,u);
%figure(2),pdeplot3D(model,'colormapdata',result.NodalSolution(:,2));
[s1]=max(model.Mesh.Nodes(1,:));
indices=find(model.Mesh.Nodes(1,:)==s1);
[maxs2]=max(model.Mesh.Nodes(2,:));
[mins2]=min(model.Mesh.Nodes(2,:));
s2=(mins2+maxs2)/2;
s3=max(model.Mesh.Nodes(3,indices));
point=[s1 s2 s3];
[mesh,e,t] = meshToPet(model.Mesh);
F = pdeInterpolant(mesh,t,u);
upoint=evaluate(F,s1,s2,s3);

function g1 = g1fun(region,state)
global parg1
g1=(parg1*1)*[0;0;-1]*10^6*ones(1,length(region.x));

function g2 = g2fun(region,state)
global parg2
g2=((parg2(1)*1.8)*[1;0;0]+(parg2(2)*1.8)*[0;1;0]+(parg2(3)*1.8)*[0;0;-1])*10^6*ones(1,length(region.x));