function [ucenter, D1]=WaveEquation(par)
%checked#1
%Input: par=input settings (x,z) for simulator.
%Output: ucenter=response defined in Section 5.1, i.e., vertical displacement at spatial coordinate (1,1) at 301 equally spaced time points within time interval [0,1.25], D1=vector of time points for functional response.

global Tx Ty beta g rho H Ex Ey Lx Ly nex ney xpts ypts indicesx indicesy eps
rho=1100;%kg/m^3
g=9.8;%m/s^2
Tx=par(1); Ty=par(1); beta=par(2);
paru0=0.1*par(3:end); parut0=zeros(1,4);

tlist = linspace(0,1.25,301)';

H=0.001; Ex=10^7; Ey=10^7; rho=rho*H;
Lx=2; Ly=2; nex=25; ney=25;
[p,e,t]=MeshGen(Lx,Ly,nex,ney);
nt=size(t,2);
% Triangle point indices
it1 = t(1,:); it2 = t(2,:); it3 = t(3,:);
% Find centroids of triangles
xpts = (p(1,it1)+p(1,it2)+p(1,it3))/3;
ypts = (p(2,it1)+p(2,it2)+p(2,it3))/3;
eps=10^-15;
indicesx=cell(1,nt); indicesy=cell(1,nt);
for i=1:nt
    indicesx{i}=find(abs(xpts-xpts(i))<=(Lx/(3*nex)+eps));    
    indicesy{i}=find(abs(ypts-ypts(i))<=(Ly/(3*ney)+eps));
end

np=size(p,2);
%pdeplot(p,e,t)
%input('press enter')

x=p(1,:)'/Lx; y=p(2,:)'/Ly;
u0=(x).*((1-x).^2).*(y).*((1-y).^2)*paru0(1)+(x.^2).*((1-x)).*(y).*((1-y).^2)*paru0(2)+...
(x).*((1-x).^2).*(y.^2).*((1-y))*paru0(3)+(x.^2).*((1-x)).*(y.^2).*((1-y))*paru0(4);
ut0=sin(2*pi*x).*sin(2*pi*y)*parut0(1)+sin(4*pi*x).*sin(2*pi*y)*parut0(2)+sin(2*pi*x).*sin(4*pi*y)*parut0(3)+sin(4*pi*x).*sin(4*pi*y)*parut0(4);
%figure(1),pdeplot(p,e,t,'XYData',u0,'ZData',u0,'FaceAlpha',0.5,'ColorMap','jet','Mesh','on')
%input('press enter')
w0=1*u0;
wt0=(beta/(2*rho))*u0+ut0;
a=-(beta^2)./(4*rho);
d=rho; f=0;
w=hyperbolic(w0,wt0,tlist,@pdebound,p,e,t,@cfunc,a,f,d,'rtol',10^-6,'atol',10^-9);

u=w.*(exp(-beta*repmat(tlist',np,1)./(2*rho)));
%figure(10),pdeplot(p,e,t,'xydata',u(:,end),'contour','on')

F = pdeInterpolant(p,t,u); 
%ucenter = evaluate(F,0,0)';
ucenter = evaluate(F,1,1)';
D1=tlist;

function [qmatrix,gmatrix,hmatrix,rmatrix] = pdebound(p,e,u,time)
ne = size(e,2); % number of edges
qmatrix = zeros(1,ne);
gmatrix = qmatrix;

hmatrix = ones(1,2*ne);
rmatrix = 0*hmatrix;

function c = cfunc(p,t,u,time)
global Tx Ty H Ex Ey Lx Ly nex ney xpts ypts eps indicesx indicesy
nt=size(t,2);
if(sum(isnan(u))>0)
    c=NaN*ones(1,nt);
    return
end
[ux,uy] = pdegrad(p,t,u);
c=zeros(2,nt);
for i=1:nt
    mx=length(indicesx{i});
    my=length(indicesy{i});
    for k=1:my
    if(abs(ypts(indicesy{i}(k))-ypts(i))<eps)
        c(1,i)=c(1,i)+sqrt(1+ux(indicesy{i}(k))^2)*(2/3*(Lx/nex));
    else
        c(1,i)=c(1,i)+sqrt(1+ux(indicesy{i}(k))^2)*(1/3*(Lx/nex));        
    end    
    end
    for k=1:mx
    if(abs(xpts(indicesx{i}(k))-xpts(i))<eps)
        c(2,i)=c(2,i)+sqrt(1+uy(indicesx{i}(k))^2)*(2/3*(Ly/ney));
    else
        c(2,i)=c(2,i)+sqrt(1+uy(indicesx{i}(k))^2)*(1/3*(Ly/ney));        
    end
    end    
end
if((sum((c(1,:)/Lx-1)>1.5))||(sum((c(2,:)/Ly-1)>1.5)>0))
    display('error')
    return
end
c(1,:)=(c(1,:)/Lx-1)*H*Ex+Tx;
c(2,:)=(c(2,:)/Ly-1)*H*Ey+Ty;