function [p, t, nNodes]=femTriangularMeshGenerator(Lx,Ly,Nx,NE)
%checked#1
    %   This function generates triangular mesh for a rectangular
    %   shape structure for finite element analysis
    %   [p t nNodes ]=femTriangularMeshGenerator(Lx,Ly,Nx,NE)
    %   p  =   x and y coordinates of each element nodes
    %   nNodes  =   Number of nodes
    %   Lx      =   width of the rectangular structure
    %   Ly      =   Height of the rectangular structure
    %   Nx      =   Number of divisions on x- axis
    %   NE      =   Number of elements  
    
    if(mod((NE/Nx),2)~=0)
        disp('error')       
    end
    
    Ny=NE/(2*Nx);   %Divisions on y axis
    
    nNodes =(Nx+1)*(Ny+1);  %No of nodes
 
    p=(fullfact([Nx+1 Ny+1])-1)./repmat([Nx Ny],nNodes,1);
    
    t=[];
    for i=1:nNodes
        if((p(i,1)<1)&&(p(i,2)<1))
            A=(abs(p(i,1)-p(:,1))<=(1/Nx+10^-15)).*(abs(p(i,2)-p(:,2))<=(1/Ny+10^-15));
            Ind1=find((A).*(p(:,1)>p(i,1)).*(p(:,2)>=p(i,2)));
            Ind2=find((A).*(p(:,1)>=p(i,1)).*(p(:,2)>p(i,2)));
            t=[t [i;Ind1] [i;Ind2]];
        end
    end

    p=p';
    p(1,:)=p(1,:)*Lx;
    p(2,:)=p(2,:)*Ly;