%function simulationforelasticity
%checked#1
%Variables: n=size of design=42, ntest=size of test set=60, 
%storedatau1=vector of MAE, MSE, APIL, and coverage for conditional linear GP model predictions of first component of multivariate response (displacement in s_1 direction), 
%storedatau2=vector of MAE, MSE, APIL, and coverage for conditional linear GP model predictions of second component of multivariate response (displacement in s_3 direction), 
%storedata2u1=vector of MAE, MSE, APIL, and coverage for standard GP model predictions of first component of multivariate response (displacement in s_1 direction), 
%storedata2u2=vector of MAE, MSE, APIL, and coverage for standard GP model predictions of second component of multivariate response (displacement in s_3 direction). 

delete(gcp('nocreate'))
parpool(1)

model = createpde(3);
importGeometry(model,'BracketWithHole.stl');

figure(100),
h = pdegplot(model,'FaceLabels','on');
view(30,30);
title('Your Face Labels')
h(1).FaceAlpha = 0.5;
openfig('MyFaceLabels.fig')

disp('Please check the two figures that have been produced. If there are discrepancies in the labels of faces labeled 4,7,11 in "My Face Labels" between the two figures, you need to change the face labels in Lines 20-22 of ThreeDimElasticity.')
disp('For example, if the face labeled 7 in the figure titled "My Face Labels" is labeled as X in the figure titled "Your Face Labels", please change 7 to X in Line 21 of ThreeDimElasticity.')
input('Press enter to continue, and press Ctrl+C to terminate.')

No=1;
Min=[150 0.15 -1 -1 -1 -1];
Max=[250 0.45 1 1 1 1];
Range=Max-Min;
d=6;
n=7*d;

Mid=(Max+Min)/2;
L=4; L2=d-L;
E=[zeros(1,L); full(speye(L))]; U=[]; Y=[];
for i=1:(L+1)
    [Y0,D1]=ThreeDimElasticity([Mid(1:L2) E(i,:)]);
    Y=[Y Y0];
    if(i>1)
        Y0=Y0-U(:,1);
    end
    U=[U Y0];
end
N=size(U,1); 

ntest=10*d;
TestSet=lhsdesign(ntest,d,'criterion','maximin');
UncodedTestSet=TestSet.*repmat(Range,ntest,1)+repmat(Min,ntest,1);
TrueTestOutput=zeros(N,ntest);
time0=zeros(ntest,1);
parfor i=1:ntest
    tic;
    [TrueTestOutput(:,i)]=ThreeDimElasticity(UncodedTestSet(i,:));
    time0(i)=toc;
end

nsim=1;
storedatau1=zeros(nsim,4);
storedatau2=zeros(nsim,4);
storedata2u1=zeros(nsim,4);
storedata2u2=zeros(nsim,4);
time1=zeros(nsim,ntest+1);
time2=zeros(nsim,ntest+1);
for run=1:nsim
    
Design=[[repmat([0.5*ones(1,L2)],L+1,1) (E+1)/2]; lhsdesign(n-L-1,d,'criterion','maximin')];
UncodedDesign=Design.*repmat(Range,n,1)+repmat(Min,n,1);
Y=[Y zeros(N,n-L-1)];
parfor i=(L+2):n
    Y(:,i)=ThreeDimElasticity(UncodedDesign(i,:));
end

N=N-1;
U=U([1 3],:); Y=Y([1 3],:); TrueTestOutput=TrueTestOutput([1 3],:);

tic
[thetaopt,miu,Phi,ResR2I,R2I]=MultiResponseGPFitElasticity2(UncodedDesign,Y,U);
time1(run,1)=toc;
Ypred=zeros(N,ntest);
UCL=zeros(N,ntest);
LCL=zeros(N,ntest);
parfor i=1:ntest
    tic
    [Ypred(:,i),UCL(:,i),LCL(:,i)]=MultiResponseGPPredictElasticity2(UncodedDesign,UncodedTestSet(i,:),thetaopt,U,Phi,ResR2I,R2I);   
    time1(run,i+1)=toc;
end

MAEu1=zeros(ntest,1);
MAEu2=zeros(ntest,1);
MSEu1=zeros(ntest,1);
MSEu2=zeros(ntest,1);
APLu1=zeros(ntest,1);
APLu2=zeros(ntest,1);
Coverageu1=zeros(ntest,1);
Coverageu2=zeros(ntest,1);
parfor i=1:ntest
    absdev=abs(Ypred(:,i)-TrueTestOutput(:,i));
    MAEu1(i)=absdev(1);
    MAEu2(i)=absdev(2); 
    MSEu1(i)=absdev(1).^2;
    MSEu2(i)=absdev(2).^2; 
    APLu1(i)=UCL(1,i)-LCL(1,i);
    APLu2(i)=UCL(2,i)-LCL(2,i);
    Coverageu1(i)=mean((LCL(1,i)<=TrueTestOutput(1,i)).*(TrueTestOutput(1,i)<=UCL(1,i)));
    Coverageu2(i)=mean((LCL(2,i)<=TrueTestOutput(2,i)).*(TrueTestOutput(2,i)<=UCL(2,i))); 
end
storedatau1(run,:)=[mean(MAEu1) mean(MSEu1) mean(APLu1) mean(Coverageu1)];
storedatau2(run,:)=[mean(MAEu2) mean(MSEu2) mean(APLu2) mean(Coverageu2)];

tic
[thetaoptNo2,alphaNo2,PhiNo2,ResR2INo2,R2INo2]=MultiResponseGPFit(Design,Y,No);
time2(run,1)=toc;
Ypred2=zeros(N,ntest);
UCL2=zeros(N,ntest);
LCL2=zeros(N,ntest);
parfor i=1:ntest
    tic
    [Ypred2(:,i),UCL2(:,i),LCL2(:,i)]=MultiResponseGPPredict(Design,No,TestSet(i,:),thetaoptNo2,alphaNo2,PhiNo2,ResR2INo2,R2INo2);     
    time2(run,i+1)=toc;
%    figure(1),hold on,plot(TrueTestOutput(:,i),Ypred(:,i),'o')
%    plot(TrueTestOutput(:,i),Ypred2(:,i),'*')
%    input('press enter')
end
MAE2u1=zeros(ntest,1);
MAE2u2=zeros(ntest,1);
MSE2u1=zeros(ntest,1);
MSE2u2=zeros(ntest,1);
APL2u1=zeros(ntest,1);
APL2u2=zeros(ntest,1);
Coverage2u1=zeros(ntest,1);
Coverage2u2=zeros(ntest,1);
parfor i=1:ntest
    absdev2=abs(Ypred2(:,i)-TrueTestOutput(:,i));
    MAE2u1(i)=absdev2(1);
    MAE2u2(i)=absdev2(2);
    MSE2u1(i)=absdev2(1).^2;
    MSE2u2(i)=absdev2(2).^2;
    APL2u1(i)=UCL2(1,i)-LCL2(1,i);
    APL2u2(i)=UCL2(2,i)-LCL2(2,i);
    Coverage2u1(i)=mean((LCL2(1,i)<=TrueTestOutput(1,i)).*(TrueTestOutput(1,i)<=UCL2(1,i)));
    Coverage2u2(i)=mean((LCL2(2,i)<=TrueTestOutput(2,i)).*(TrueTestOutput(2,i)<=UCL2(2,i)));
end
storedata2u1(run,:)=[mean(MAE2u1) mean(MSE2u1) mean(APL2u1) mean(Coverage2u1)];
storedata2u2(run,:)=[mean(MAE2u2) mean(MSE2u2) mean(APL2u2) mean(Coverage2u2)];

end

disp({'MAE, MSE, APIL, and coverage (of nominal 98% intervals) for response y_1 given by conditional linear GP model'})
disp(storedatau1)
disp({'MAE, MSE, APIL, and coverage (of nominal 98% intervals) for response y_1 given by standard GP model'})
disp(storedata2u1)

disp({'MAE, MSE, APIL, and coverage (of nominal 98% intervals) for response y_2 given by conditional linear GP model'})
disp(storedatau2) 
disp({'MAE, MSE, APIL, and coverage (of nominal 98% intervals) for response y_2 given by standard GP model'})
disp(storedata2u2)
